#include <Wire.h>
#include <LSM303.h>
#include <Ultrasonic.h>


// Моторы подключаются к клеммам M1+, M1-, M2+, M2- Motor Sheild 1
// Motor shield использует четыре контакта 4, 5, 6, 7 для управления моторами 
// 4 и 7 — для направления, 5 и 6 — для скорости
#define SPEED_1      5 
#define DIR_1        4
#define SPEED_2      6
#define DIR_2        7

// Кран и помпа подключается к клеммам M1+, M1-, M2+, M2- Motor Sheild 2
#define DIR_PIN_K     8 // этот пин отвечает за направление
#define SPEED_PIN_K   9 // этот пин отвечает за скорость
#define DIR_PIN_P   2 // земля
#define SPEED_PIN_P   3 // этот пин отвечает за напор

// Присваиваем переменные для дальномеров 
// датчика пламени (А1)
#define RIGHT_RANGEFINDER A0
#define FRONT_RANGEFINDER A1
#define LEFT_RANGEFINDER A2
#define FLAME A3

enum Rotate{NONE = 0, LEFT = 1, RIGHT = 2};

LSM303 compass;
Ultrasonic ultrasonic(11,12);

int RUN=160;
int SPEED_K=130;
int mode = 0;
int main_direction = 0;

//Процедуры управления помпой
void PumpOff()
{
  analogWrite(SPEED_PIN_P, 0);
} 
void PumpOn()
{
  analogWrite(SPEED_PIN_P, 1023);
} 

//Процедуры управления краном
void kranUp()
{
  analogWrite(SPEED_PIN_K, SPEED_K);
  digitalWrite(DIR_PIN_K, LOW);
} 
void kranDown()
{
  analogWrite(SPEED_PIN_K, SPEED_K-25);
  digitalWrite(DIR_PIN_K, HIGH);
}
void kranStop()
{
  analogWrite(SPEED_PIN_K, 0);
  digitalWrite(DIR_PIN_K, HIGH);
}

//описываем процедуры движения
//процедура движения вперед
void go()
{
  //задаем скорость вращения моторов
  analogWrite(SPEED_1, RUN);
  analogWrite(SPEED_2, RUN+10);
}
//остановка
void stop()
{
  analogWrite(SPEED_1, 0);
  analogWrite(SPEED_2, 0);
}
//вперёд
void goForward()
{
  digitalWrite(DIR_2, HIGH);
  digitalWrite(DIR_1, LOW);
  go();
}

//поворот направо
void turnRight(int m = 1)
{
  digitalWrite(DIR_1, HIGH);
  digitalWrite(DIR_2, HIGH);
  analogWrite(SPEED_1, 180);
  analogWrite(SPEED_2, 180);
}

//поворот налево
void turnLeft(int m = 1)
{
  digitalWrite(DIR_1, LOW);
  digitalWrite(DIR_2, LOW);
  analogWrite(SPEED_1, 180);
  analogWrite(SPEED_2, 180);
}



//Константы расстояний
const int dist_B       = 80; //sm, all, visibility of obstacles
const int dist_A       = 20; //sm, left & right, visibility of wall
const int dist_Z       = 20; //sm, left & right, swivel area
const int dist_Z_front = 30; //sm, front, swivel area

//переменные объезда препятствия
int priority_turnaround = 0; // 0 - default (uncertainty, right), 1 - left, 2 - right
int deviation = 0; // 0 - need not rotate (default), 1 - need left rotate, 2 - need right rotate;
int direction_of_rotation = 0; // 0 - need none rotate, 1 - need left rotate, 2 - need right rotate;

//function of rangefinder
int read_sharp_2Y0A02(int analog_input = FRONT_RANGEFINDER) //Front rangefinder
{
  float volts = analogRead(analog_input)*0.0048828125;   // считываем значение сенсора и переводим в напряжение
  float distance = 65*pow(volts, -1.10);
  return distance;
}

int read_sharp_2Y0A21(int analog_input) //Right and Left rangefinder
{
  float volts = analogRead(analog_input)*0.0048828125;   // считываем значение сенсора и переводим в напряжение
  float distance = 27.5*pow(volts, -1.03);
  return distance;
}

void navigation()
{
  int L = read_sharp_2Y0A21(LEFT_RANGEFINDER);
  int R = read_sharp_2Y0A21(RIGHT_RANGEFINDER);
  int F = read_sharp_2Y0A02();
  
  Serial.print("L = "); Serial.println(L);
  Serial.print("F = "); Serial.println(F);
  Serial.print("R = "); Serial.println(R);
  
  /*if (deviation == LEFT && R > dist_A) {
    direction_of_rotation = RIGHT;
    delay(200);
    return;
  }
  if (deviation == RIGHT && L > dist_A) {
    direction_of_rotation = LEFT;
    delay(200);
    return;
  }*/
  
  direction_of_rotation = NONE;
  if (F < dist_Z_front) {
       direction_of_rotation = (priority_turnaround == NONE)?RIGHT:priority_turnaround;
  } else if (L < dist_Z && R > dist_Z) {
    direction_of_rotation = RIGHT;
  } else if (R < dist_Z && L > dist_Z) {
    direction_of_rotation = LEFT;
  } else {
     if (L > R) {
       priority_turnaround = LEFT;
     } else if (L < R) {
       priority_turnaround = RIGHT;
     }
  }      
}

void get_deviation(){
    float current_direction; 
    compass.read();
    current_direction = compass.heading();
    
    
    Serial.print("current_direction - main_direction = "); Serial.println(current_direction - main_direction);	//Serial.print("current_direction = "); Serial.println(current_direction);
    
    if ((current_direction - main_direction) < -180) {
      deviation = RIGHT;
    } else if ((current_direction - main_direction) < -15)
    {
      deviation = LEFT;
    } else if ((current_direction - main_direction) < 15)
    {
      deviation = NONE;
    } else if ((current_direction - main_direction) < 180)
    {
      deviation = RIGHT;
    }else
    {
      deviation = LEFT;
    }
}


void setup()
{
  Serial.begin(9600);
  Wire.begin();
  
  //initial compas
  compass.init();
  compass.enableDefault();  
  compass.m_min = (LSM303::vector<int16_t>){ -92,   -242, -11271};
  compass.m_max = (LSM303::vector<int16_t>){ +1642,  +1614,  -8871};
  compass.read();
  main_direction = compass.heading((LSM303::vector<int>){1, 0, 0});
  Serial.print("main_direction = ");Serial.println(main_direction);
  
  //начальные установки для аналоговых датчиков
  for(int i = 2; i <= 9; i++)     
    pinMode(i, OUTPUT);     

  digitalWrite(DIR_PIN_P, LOW);  
  pinMode(LEFT_RANGEFINDER, INPUT); 
  pinMode(FRONT_RANGEFINDER, INPUT); 
  pinMode(RIGHT_RANGEFINDER, INPUT); 
  pinMode(FLAME, INPUT); 
  srand(compass.m.x);
  
  delay (200); 
}

int is_start = 0;
int f_mode = 0;
int detected_mode = 0;
unsigned long timer_rotate = 0; 
unsigned long patrul_timer = 0;

void loop()
{ 
  //stop();
        if(is_start == 0)
        {
        	compass.read();
        	main_direction = compass.heading();
        	is_start = 1;
        	patrul_timer = millis();
        }
	int RT;
	RT = analogRead (FLAME);
	//Serial.print("rt = "); Serial.println(RT);
	if (RT > 200)
	{
		compass.read();
		main_direction = compass.heading();
		f_mode = 1;
		detected_mode = 0;
	}
	
	if (f_mode == 0) {	
	  //Serial.print("main_direction = "); Serial.println(main_direction);
	  if (millis() - patrul_timer > 10000 && detected_mode == 0) {
		Serial.print("main_direction = "); Serial.println(main_direction);
		Serial.print("(main_direction+90)%360 = "); Serial.println((main_direction+90)%360);
		main_direction = (main_direction+90)%360;
		detected_mode = 1;
	  }
	  
		
	  
	  get_deviation();
	  Serial.print("detected_mode = "); Serial.println(detected_mode);
	  
	  if (detected_mode > 0) {
		if (deviation == NONE)
		{
			main_direction = (main_direction+90)%360;
			detected_mode == (detected_mode == 4)?0:(detected_mode+1);
			patrul_timer = millis();
		} 
	  }
	  
	  get_deviation();
	  navigation();
	  if (direction_of_rotation == NONE) {
		if (deviation != NONE) {
		 if ( millis() - timer_rotate >= 2000){
			if (deviation == LEFT) {
			  turnRight();
			}
			if (deviation == RIGHT) {
			  turnLeft();
			}
		 }
		 else {
			goForward();
			//stop();
		 }
		} else {
			goForward();
			//stop();
		 timer_rotate = millis();
		}
	  } else{ 
                //Serial.println("Препятствие");
                if (direction_of_rotation == LEFT) 
		    turnLeft();
                else if (direction_of_rotation == RIGHT) 
		    turnRight();
		timer_rotate = millis();
	  }
	} else {
                Serial.println("Пожар");
		int RT;
		RT = analogRead (FLAME);  
	        //Serial.print("rt = "); Serial.println(RT);                                                        
		if (RT < 220)
		{
			f_mode = 0;
		} else 
		{
	                Serial.print("ultrasonic = "); Serial.println(ultrasonic.Ranging(CM));
			if (ultrasonic.Ranging(CM) < 20 && RT > 240)
			{
                                stop();
				kranUp();
				delay(1000);
				unsigned long time_up = millis();
				//while (ultrasonic.Ranging(CM) < 15);
				kranStop();
				time_up = millis() - time_up;
				PumpOn();
				delay(500);
				//while (analogRead (FLAME) > 120);
				kranDown();
				delay(1000);
				kranStop();
                                PumpOff();
				f_mode = 0;
			}
			else {
				goForward();
			}
		}
	}
  
}



